"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.composeKeyedIntersection = exports.isEquality = exports.equality = exports.isDisjoint = exports.anonymousDisjoint = exports.IntersectionState = exports.stringifyRange = exports.disjointDescriptionWriters = exports.undefinedOperandsMessage = exports.composeIntersection = void 0;
const errors_js_1 = require("../utils/errors.js");
const generics_js_1 = require("../utils/generics.js");
const paths_js_1 = require("../utils/paths.js");
const serialize_js_1 = require("../utils/serialize.js");
const composeIntersection = (reducer) => ((l, r, state) => l === undefined
    ? r === undefined
        ? (0, errors_js_1.throwInternalError)(exports.undefinedOperandsMessage)
        : r
    : r === undefined
        ? l
        : reducer(l, r, state));
exports.composeIntersection = composeIntersection;
exports.undefinedOperandsMessage = `Unexpected operation two undefined operands`;
exports.disjointDescriptionWriters = {
    domain: ({ l, r }) => `${l.join(", ")} and ${r.join(", ")}`,
    range: ({ l, r }) => `${(0, exports.stringifyRange)(l)} and ${(0, exports.stringifyRange)(r)}`,
    class: ({ l, r }) => `classes ${typeof l === "string" ? l : l.name} and ${typeof r === "string" ? r : r.name}`,
    tupleLength: ({ l, r }) => `tuples of length ${l} and ${r}`,
    value: ({ l, r }) => `literal values ${(0, serialize_js_1.stringify)(l)} and ${(0, serialize_js_1.stringify)(r)}`,
    leftAssignability: ({ l, r }) => `literal value ${(0, serialize_js_1.stringify)(l.value)} and ${(0, serialize_js_1.stringify)(r)}`,
    rightAssignability: ({ l, r }) => `literal value ${(0, serialize_js_1.stringify)(r.value)} and ${(0, serialize_js_1.stringify)(l)}`,
    union: ({ l, r }) => `branches ${(0, serialize_js_1.stringify)(l)} and branches ${(0, serialize_js_1.stringify)(r)}`
};
const stringifyRange = (range) => "limit" in range
    ? `the range of exactly ${range.limit}`
    : range.min
        ? range.max
            ? `the range bounded by ${range.min.comparator}${range.min.limit} and ${range.max.comparator}${range.max.limit}`
            : `${range.min.comparator}${range.min.limit}`
        : range.max
            ? `${range.max.comparator}${range.max.limit}`
            : "the unbounded range";
exports.stringifyRange = stringifyRange;
class IntersectionState {
    type;
    lastOperator;
    path = new paths_js_1.Path();
    domain;
    #disjoints = {};
    constructor(type, lastOperator) {
        this.type = type;
        this.lastOperator = lastOperator;
    }
    get disjoints() {
        return this.#disjoints;
    }
    addDisjoint(kind, l, r) {
        this.#disjoints[`${this.path}`] = { kind, l, r };
        return empty;
    }
}
exports.IntersectionState = IntersectionState;
const empty = Symbol("empty");
const anonymousDisjoint = () => empty;
exports.anonymousDisjoint = anonymousDisjoint;
const isDisjoint = (result) => result === empty;
exports.isDisjoint = isDisjoint;
const equal = Symbol("equal");
const equality = () => equal;
exports.equality = equality;
const isEquality = (result) => result === equal;
exports.isEquality = isEquality;
const composeKeyedIntersection = (reducer, config) => (l, r, state) => {
    const result = {};
    const keys = (0, generics_js_1.objectKeysOf)({ ...l, ...r });
    let lImpliesR = true;
    let rImpliesL = true;
    for (const k of keys) {
        const keyResult = typeof reducer === "function"
            ? reducer(k, l[k], r[k], state)
            : reducer[k](l[k], r[k], state);
        if ((0, exports.isEquality)(keyResult)) {
            if (l[k] !== undefined) {
                result[k] = l[k];
            }
        }
        else if ((0, exports.isDisjoint)(keyResult)) {
            if (config.onEmpty === "omit") {
                lImpliesR = false;
                rImpliesL = false;
            }
            else {
                return empty;
            }
        }
        else {
            if (keyResult !== undefined) {
                result[k] = keyResult;
            }
            lImpliesR &&= keyResult === l[k];
            rImpliesL &&= keyResult === r[k];
        }
    }
    return lImpliesR ? (rImpliesL ? (0, exports.equality)() : l) : rImpliesL ? r : result;
};
exports.composeKeyedIntersection = composeKeyedIntersection;
