"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.toArrayNode = exports.resolutionExtendsDomain = exports.isLiteralNode = exports.flattenTypeNode = exports.flattenNode = exports.flattenType = exports.rootUnion = exports.rootIntersection = exports.nodeIntersection = exports.isConfigNode = void 0;
const intersection_js_1 = require("../parse/ast/intersection.js");
const errors_js_1 = require("../utils/errors.js");
const generics_js_1 = require("../utils/generics.js");
const paths_js_1 = require("../utils/paths.js");
const compose_js_1 = require("./compose.js");
const predicate_js_1 = require("./predicate.js");
const props_js_1 = require("./rules/props.js");
const isConfigNode = (node) => "config" in node;
exports.isConfigNode = isConfigNode;
const nodeIntersection = (l, r, state) => {
    state.domain = undefined;
    const lDomains = state.type.scope.resolveTypeNode(l);
    const rDomains = state.type.scope.resolveTypeNode(r);
    const result = typeNodeIntersection(lDomains, rDomains, state);
    if (typeof result === "object" && !(0, generics_js_1.hasKeys)(result)) {
        return (0, generics_js_1.hasKeys)(state.disjoints)
            ? (0, compose_js_1.anonymousDisjoint)()
            : state.addDisjoint("domain", (0, generics_js_1.objectKeysOf)(lDomains), (0, generics_js_1.objectKeysOf)(rDomains));
    }
    return result === lDomains ? l : result === rDomains ? r : result;
};
exports.nodeIntersection = nodeIntersection;
const typeNodeIntersection = (0, compose_js_1.composeKeyedIntersection)((domain, l, r, context) => {
    if (l === undefined) {
        return r === undefined
            ? (0, errors_js_1.throwInternalError)(compose_js_1.undefinedOperandsMessage)
            : undefined;
    }
    if (r === undefined) {
        return undefined;
    }
    return (0, predicate_js_1.predicateIntersection)(domain, l, r, context);
}, { onEmpty: "omit" });
const rootIntersection = (l, r, type) => {
    const state = new compose_js_1.IntersectionState(type, "&");
    const result = (0, exports.nodeIntersection)(l, r, state);
    return (0, compose_js_1.isDisjoint)(result)
        ? (0, errors_js_1.throwParseError)((0, intersection_js_1.compileDisjointReasonsMessage)(state.disjoints))
        : (0, compose_js_1.isEquality)(result)
            ? l
            : result;
};
exports.rootIntersection = rootIntersection;
const rootUnion = (l, r, type) => {
    const lDomains = type.scope.resolveTypeNode(l);
    const rDomains = type.scope.resolveTypeNode(r);
    const result = {};
    const domains = (0, generics_js_1.objectKeysOf)({ ...lDomains, ...rDomains });
    for (const domain of domains) {
        result[domain] = (0, generics_js_1.hasKey)(lDomains, domain)
            ? (0, generics_js_1.hasKey)(rDomains, domain)
                ? (0, predicate_js_1.predicateUnion)(domain, lDomains[domain], rDomains[domain], type)
                : lDomains[domain]
            : (0, generics_js_1.hasKey)(rDomains, domain)
                ? rDomains[domain]
                : (0, errors_js_1.throwInternalError)(compose_js_1.undefinedOperandsMessage);
    }
    return result;
};
exports.rootUnion = rootUnion;
const hasImpliedDomain = (flatPredicate) => flatPredicate[0] &&
    (flatPredicate[0][0] === "value" || flatPredicate[0][0] === "class");
const flattenType = (type) => {
    const ctx = {
        type,
        path: new paths_js_1.Path(),
        lastDomain: "undefined"
    };
    return (0, exports.flattenNode)(type.node, ctx);
};
exports.flattenType = flattenType;
const flattenNode = (node, ctx) => {
    if (typeof node === "string") {
        return ctx.type.scope.resolve(node).flat;
    }
    const hasConfig = (0, exports.isConfigNode)(node);
    const flattenedTypeNode = (0, exports.flattenTypeNode)(hasConfig ? node.node : node, ctx);
    return hasConfig
        ? [
            [
                "config",
                {
                    config: (0, generics_js_1.entriesOf)(node.config),
                    node: flattenedTypeNode
                }
            ]
        ]
        : flattenedTypeNode;
};
exports.flattenNode = flattenNode;
const flattenTypeNode = (node, ctx) => {
    const domains = (0, generics_js_1.objectKeysOf)(node);
    if (domains.length === 1) {
        const domain = domains[0];
        const predicate = node[domain];
        if (predicate === true) {
            return domain;
        }
        ctx.lastDomain = domain;
        const flatPredicate = (0, predicate_js_1.flattenPredicate)(predicate, ctx);
        return hasImpliedDomain(flatPredicate)
            ? flatPredicate
            : [["domain", domain], ...flatPredicate];
    }
    const result = {};
    for (const domain of domains) {
        ctx.lastDomain = domain;
        result[domain] = (0, predicate_js_1.flattenPredicate)(node[domain], ctx);
    }
    return [["domains", result]];
};
exports.flattenTypeNode = flattenTypeNode;
const isLiteralNode = (node, domain) => {
    return ((0, exports.resolutionExtendsDomain)(node, domain) &&
        (0, predicate_js_1.isLiteralCondition)(node[domain]));
};
exports.isLiteralNode = isLiteralNode;
const resolutionExtendsDomain = (resolution, domain) => {
    const domains = (0, generics_js_1.objectKeysOf)(resolution);
    return domains.length === 1 && domains[0] === domain;
};
exports.resolutionExtendsDomain = resolutionExtendsDomain;
const toArrayNode = (node) => ({
    object: {
        class: Array,
        props: {
            [props_js_1.mappedKeys.index]: node
        }
    }
});
exports.toArrayNode = toArrayNode;
