"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.isLiteralCondition = exports.flattenPredicate = exports.predicateUnion = exports.predicateIntersection = exports.comparePredicates = void 0;
const generics_js_1 = require("../utils/generics.js");
const objectKinds_js_1 = require("../utils/objectKinds.js");
const branch_js_1 = require("./branch.js");
const compose_js_1 = require("./compose.js");
const discriminate_js_1 = require("./discriminate.js");
const emptyRulesIfTrue = (predicate) => (predicate === true ? {} : predicate);
const comparePredicates = (l, r, context) => {
    if (l === true && r === true) {
        return (0, compose_js_1.equality)();
    }
    if (!(0, objectKinds_js_1.isArray)(l) && !(0, objectKinds_js_1.isArray)(r)) {
        const result = (0, branch_js_1.branchIntersection)(emptyRulesIfTrue(l), emptyRulesIfTrue(r), context);
        return result === l ? l : result === r ? r : result;
    }
    const lBranches = (0, generics_js_1.listFrom)(emptyRulesIfTrue(l));
    const rBranches = (0, generics_js_1.listFrom)(emptyRulesIfTrue(r));
    const comparison = (0, branch_js_1.compareBranches)(lBranches, rBranches, context);
    if (comparison.equalities.length === lBranches.length &&
        comparison.equalities.length === rBranches.length) {
        return (0, compose_js_1.equality)();
    }
    if (comparison.lExtendsR.length + comparison.equalities.length ===
        lBranches.length) {
        return l;
    }
    if (comparison.rExtendsL.length + comparison.equalities.length ===
        rBranches.length) {
        return r;
    }
    return comparison;
};
exports.comparePredicates = comparePredicates;
const predicateIntersection = (domain, l, r, state) => {
    state.domain = domain;
    const comparison = (0, exports.comparePredicates)(l, r, state);
    if (!(0, branch_js_1.isBranchComparison)(comparison)) {
        return comparison;
    }
    const resultBranches = [
        ...comparison.distinctIntersections,
        ...comparison.equalities.map((indices) => comparison.lBranches[indices[0]]),
        ...comparison.lExtendsR.map((lIndex) => comparison.lBranches[lIndex]),
        ...comparison.rExtendsL.map((rIndex) => comparison.rBranches[rIndex])
    ];
    if (resultBranches.length === 0) {
        state.addDisjoint("union", comparison.lBranches, comparison.rBranches);
    }
    return resultBranches.length === 1 ? resultBranches[0] : resultBranches;
};
exports.predicateIntersection = predicateIntersection;
const predicateUnion = (domain, l, r, type) => {
    const state = new compose_js_1.IntersectionState(type, "|");
    const comparison = (0, exports.comparePredicates)(l, r, state);
    if (!(0, branch_js_1.isBranchComparison)(comparison)) {
        return (0, compose_js_1.isEquality)(comparison) || comparison === l
            ? r
            : comparison === r
                ? l
                : // if a boolean has multiple branches, neither of which is a
                    // subtype of the other, it consists of two opposite literals
                    // and can be simplified to a non-literal boolean.
                    domain === "boolean"
                        ? true
                        : [emptyRulesIfTrue(l), emptyRulesIfTrue(r)];
    }
    const resultBranches = [
        ...comparison.lBranches.filter((_, lIndex) => !comparison.lExtendsR.includes(lIndex) &&
            !comparison.equalities.some((indexPair) => indexPair[0] === lIndex)),
        ...comparison.rBranches.filter((_, rIndex) => !comparison.rExtendsL.includes(rIndex) &&
            !comparison.equalities.some((indexPair) => indexPair[1] === rIndex))
    ];
    return resultBranches.length === 1 ? resultBranches[0] : resultBranches;
};
exports.predicateUnion = predicateUnion;
const flattenPredicate = (predicate, context) => {
    if (predicate === true) {
        return [];
    }
    return (0, objectKinds_js_1.isArray)(predicate)
        ? (0, discriminate_js_1.flattenBranches)(predicate, context)
        : (0, branch_js_1.flattenBranch)(predicate, context);
};
exports.flattenPredicate = flattenPredicate;
const isLiteralCondition = (predicate) => typeof predicate === "object" && "value" in predicate;
exports.isLiteralCondition = isLiteralCondition;
