"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.compareStrictness = exports.checkBound = exports.flattenRange = exports.rangeIntersection = exports.isEqualityRange = exports.maxComparators = exports.minComparators = void 0;
const data_js_1 = require("../../utils/data.js");
const compose_js_1 = require("../compose.js");
exports.minComparators = {
    ">": true,
    ">=": true
};
exports.maxComparators = {
    "<": true,
    "<=": true
};
const isEqualityRange = (range) => "comparator" in range;
exports.isEqualityRange = isEqualityRange;
exports.rangeIntersection = (0, compose_js_1.composeIntersection)((l, r, state) => {
    if ((0, exports.isEqualityRange)(l)) {
        if ((0, exports.isEqualityRange)(r)) {
            return l.limit === r.limit
                ? (0, compose_js_1.equality)()
                : state.addDisjoint("range", l, r);
        }
        return rangeAllows(r, l.limit) ? l : state.addDisjoint("range", l, r);
    }
    if ((0, exports.isEqualityRange)(r)) {
        return rangeAllows(l, r.limit) ? r : state.addDisjoint("range", l, r);
    }
    const stricterMin = (0, exports.compareStrictness)("min", l.min, r.min);
    const stricterMax = (0, exports.compareStrictness)("max", l.max, r.max);
    if (stricterMin === "l") {
        if (stricterMax === "r") {
            return (0, exports.compareStrictness)("min", l.min, r.max) === "l"
                ? state.addDisjoint("range", l, r)
                : {
                    min: l.min,
                    max: r.max
                };
        }
        return l;
    }
    if (stricterMin === "r") {
        if (stricterMax === "l") {
            return (0, exports.compareStrictness)("max", l.max, r.min) === "l"
                ? state.addDisjoint("range", l, r)
                : {
                    min: r.min,
                    max: l.max
                };
        }
        return r;
    }
    return stricterMax === "l" ? l : stricterMax === "r" ? r : (0, compose_js_1.equality)();
});
const rangeAllows = (range, n) => (0, exports.isEqualityRange)(range)
    ? n === range.limit
    : minAllows(range.min, n) && maxAllows(range.max, n);
const minAllows = (min, n) => !min || n > min.limit || (n === min.limit && !isExclusive(min.comparator));
const maxAllows = (max, n) => !max || n < max.limit || (n === max.limit && !isExclusive(max.comparator));
const flattenRange = (entries, range, ctx) => {
    const units = ctx.lastDomain === "string"
        ? "characters"
        : ctx.lastDomain === "object"
            ? "items long"
            : undefined;
    if ((0, exports.isEqualityRange)(range)) {
        return entries.push(["bound", units ? { ...range, units } : range]);
    }
    if (range.min) {
        entries.push(["bound", units ? { ...range.min, units } : range.min]);
    }
    if (range.max) {
        entries.push(["bound", units ? { ...range.max, units } : range.max]);
    }
};
exports.flattenRange = flattenRange;
const checkBound = (bound, state) => comparatorCheckers[bound.comparator]((0, data_js_1.sizeOf)(state.data), bound.limit) ||
    !state.problems.add("bound", bound);
exports.checkBound = checkBound;
const comparatorCheckers = {
    "<": (size, limit) => size < limit,
    ">": (size, limit) => size > limit,
    "<=": (size, limit) => size <= limit,
    ">=": (size, limit) => size >= limit,
    "==": (size, limit) => size === limit
};
const compareStrictness = (kind, l, r) => !l
    ? !r
        ? "="
        : "r"
    : !r
        ? "l"
        : l.limit === r.limit
            ? isExclusive(l.comparator)
                ? isExclusive(r.comparator)
                    ? "="
                    : "l"
                : isExclusive(r.comparator)
                    ? "r"
                    : "="
            : kind === "min"
                ? l.limit > r.limit
                    ? "l"
                    : "r"
                : l.limit < r.limit
                    ? "l"
                    : "r";
exports.compareStrictness = compareStrictness;
const isExclusive = (comparator) => comparator.length === 1;
