"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.literalSatisfiesRules = exports.precedenceMap = exports.flattenRules = exports.narrowableRulesIntersection = exports.rulesIntersection = void 0;
const generics_js_1 = require("../../utils/generics.js");
const compose_js_1 = require("../compose.js");
const class_js_1 = require("./class.js");
const collapsibleSet_js_1 = require("./collapsibleSet.js");
const divisor_js_1 = require("./divisor.js");
const props_js_1 = require("./props.js");
const range_js_1 = require("./range.js");
const regex_js_1 = require("./regex.js");
const rulesIntersection = (l, r, state) => "value" in l
    ? "value" in r
        ? l.value === r.value
            ? (0, compose_js_1.equality)()
            : state.addDisjoint("value", l.value, r.value)
        : (0, exports.literalSatisfiesRules)(l.value, r, state)
            ? l
            : state.addDisjoint("leftAssignability", l, r)
    : "value" in r
        ? (0, exports.literalSatisfiesRules)(r.value, l, state)
            ? r
            : state.addDisjoint("rightAssignability", l, r)
        : (0, exports.narrowableRulesIntersection)(l, r, state);
exports.rulesIntersection = rulesIntersection;
const narrowIntersection = (0, compose_js_1.composeIntersection)(collapsibleSet_js_1.collapsibleListUnion);
exports.narrowableRulesIntersection = (0, compose_js_1.composeKeyedIntersection)({
    divisor: divisor_js_1.divisorIntersection,
    regex: regex_js_1.regexIntersection,
    props: props_js_1.propsIntersection,
    class: class_js_1.classIntersection,
    range: range_js_1.rangeIntersection,
    narrow: narrowIntersection
}, { onEmpty: "bubble" });
const flattenRules = (rules, ctx) => {
    const entries = [];
    let k;
    for (k in rules) {
        ruleFlatteners[k](entries, rules[k], ctx);
    }
    // Some entries with the same precedence, e.g. morphs flattened from a list,
    // rely on the fact that JS's builtin sort is stable to behave as expected
    // when traversed:
    // https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Array/sort
    return entries.sort((l, r) => exports.precedenceMap[l[0]] - exports.precedenceMap[r[0]]);
};
exports.flattenRules = flattenRules;
const ruleFlatteners = {
    regex: (entries, rule) => {
        for (const source of (0, generics_js_1.listFrom)(rule)) {
            entries.push(["regex", source]);
        }
    },
    divisor: (entries, rule) => {
        entries.push(["divisor", rule]);
    },
    range: range_js_1.flattenRange,
    class: (entries, rule) => {
        entries.push(["class", rule]);
    },
    props: props_js_1.flattenProps,
    narrow: (entries, rule) => {
        for (const narrow of (0, generics_js_1.listFrom)(rule)) {
            entries.push(["narrow", narrow]);
        }
    },
    value: (entries, rule) => {
        entries.push(["value", rule]);
    }
};
exports.precedenceMap = {
    // Config: Applies before any checks
    config: -1,
    // Critical: No other checks are performed if these fail
    domain: 0,
    value: 0,
    domains: 0,
    branches: 0,
    switch: 0,
    alias: 0,
    class: 0,
    // Shallow: All shallow checks will be performed even if one or more fail
    regex: 1,
    divisor: 1,
    bound: 1,
    // Prerequisite: These are deep checks with special priority, e.g. the
    // length of a tuple, which causes other deep props not to be checked if it
    // is invalid
    prerequisiteProp: 2,
    // Deep: Performed if all shallow checks pass, even if one or more deep checks fail
    distilledProps: 3,
    strictProps: 3,
    requiredProp: 3,
    optionalProp: 3,
    indexProp: 3,
    // Narrow: Only performed if all shallow and deep checks pass
    narrow: 4,
    // Morph: Only performed if all validation passes
    morph: 5
};
const literalSatisfiesRules = (data, rules, state) => !state.type.scope.type(["node", { [state.domain]: rules }])(data).problems;
exports.literalSatisfiesRules = literalSatisfiesRules;
