"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.distributeFunctionToNode = exports.writeMalformedDistributableFunctionMessage = void 0;
const domains_js_1 = require("../../utils/domains.js");
const errors_js_1 = require("../../utils/errors.js");
const generics_js_1 = require("../../utils/generics.js");
const serialize_js_1 = require("../../utils/serialize.js");
const writeMalformedDistributableFunctionMessage = (def) => `Expected a Function or Record<Domain, Function> operand (${(0, serialize_js_1.stringify)(def)} was invalid)`;
exports.writeMalformedDistributableFunctionMessage = writeMalformedDistributableFunctionMessage;
const distributeFunctionToNode = (distributableFunction, node, ctx, ruleKey) => {
    const domains = (0, generics_js_1.objectKeysOf)(node);
    if (!(0, domains_js_1.hasDomain)(distributableFunction, "object")) {
        return (0, errors_js_1.throwParseError)((0, exports.writeMalformedDistributableFunctionMessage)(distributableFunction));
    }
    const distributed = {};
    if (typeof distributableFunction === "function") {
        const domainFunction = {
            [ruleKey]: distributableFunction
        };
        for (const domain of domains) {
            distributed[domain] = domainFunction;
        }
    }
    else {
        for (const domain of domains) {
            if (distributableFunction[domain] === undefined) {
                continue;
            }
            const functionInDomain = {
                [ruleKey]: distributableFunction[domain]
            };
            if (typeof functionInDomain[ruleKey] !== "function") {
                return (0, errors_js_1.throwParseError)((0, exports.writeMalformedDistributableFunctionMessage)(functionInDomain));
            }
            distributed[domain] = functionInDomain;
        }
    }
    return distributed;
};
exports.distributeFunctionToNode = distributeFunctionToNode;
