"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.writeMalformedMorphExpressionMessage = exports.parseMorphTuple = void 0;
const branch_js_1 = require("../../nodes/branch.js");
const errors_js_1 = require("../../utils/errors.js");
const objectKinds_js_1 = require("../../utils/objectKinds.js");
const serialize_js_1 = require("../../utils/serialize.js");
const definition_js_1 = require("../definition.js");
const parseMorphTuple = (def, ctx) => {
    if (typeof def[2] !== "function") {
        return (0, errors_js_1.throwParseError)((0, exports.writeMalformedMorphExpressionMessage)(def[2]));
    }
    const node = (0, definition_js_1.parseDefinition)(def[0], ctx);
    const resolution = ctx.type.scope.resolveTypeNode(node);
    const morph = def[2];
    ctx.type.includesMorph = true;
    let domain;
    const result = {};
    for (domain in resolution) {
        const predicate = resolution[domain];
        if (predicate === true) {
            result[domain] = { rules: {}, morph };
        }
        else if (typeof predicate === "object") {
            result[domain] = (0, objectKinds_js_1.isArray)(predicate)
                ? predicate.map((branch) => applyMorph(branch, morph))
                : applyMorph(predicate, morph);
        }
        else {
            (0, errors_js_1.throwInternalError)(`Unexpected predicate value for domain '${domain}': ${(0, serialize_js_1.stringify)(predicate)}`);
        }
    }
    return result;
};
exports.parseMorphTuple = parseMorphTuple;
const applyMorph = (branch, morph) => (0, branch_js_1.isTransformationBranch)(branch)
    ? {
        ...branch,
        morph: branch.morph
            ? Array.isArray(branch.morph)
                ? [...branch.morph, morph]
                : [branch.morph, morph]
            : morph
    }
    : {
        rules: branch,
        morph
    };
const writeMalformedMorphExpressionMessage = (value) => `Morph expression requires a function following '|>' (was ${typeof value})`;
exports.writeMalformedMorphExpressionMessage = writeMalformedMorphExpressionMessage;
