"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.writeMalformedFunctionalExpressionMessage = exports.parseTuple = void 0;
const node_js_1 = require("../../nodes/node.js");
const domains_js_1 = require("../../utils/domains.js");
const errors_js_1 = require("../../utils/errors.js");
const definition_js_1 = require("../definition.js");
const unenclosed_js_1 = require("../string/shift/operand/unenclosed.js");
const config_js_1 = require("./config.js");
const keyof_js_1 = require("./keyof.js");
const morph_js_1 = require("./morph.js");
const narrow_js_1 = require("./narrow.js");
const parseTuple = (def, ctx) => {
    if (isIndexOneExpression(def)) {
        return indexOneParsers[def[1]](def, ctx);
    }
    if (isIndexZeroExpression(def)) {
        return prefixParsers[def[0]](def, ctx);
    }
    const props = {
        //  length is created as a prerequisite prop, ensuring if it is invalid,
        //  no other props will be checked, which is usually desirable for tuple
        //  definitions.
        length: ["!", { number: { value: def.length } }]
    };
    for (let i = 0; i < def.length; i++) {
        ctx.path.push(`${i}`);
        props[i] = (0, definition_js_1.parseDefinition)(def[i], ctx);
        ctx.path.pop();
    }
    return {
        object: {
            class: Array,
            props
        }
    };
};
exports.parseTuple = parseTuple;
const parseBranchTuple = (def, ctx) => {
    if (def[2] === undefined) {
        return (0, errors_js_1.throwParseError)((0, unenclosed_js_1.writeMissingRightOperandMessage)(def[1], ""));
    }
    const l = (0, definition_js_1.parseDefinition)(def[0], ctx);
    const r = (0, definition_js_1.parseDefinition)(def[2], ctx);
    return def[1] === "&"
        ? (0, node_js_1.rootIntersection)(l, r, ctx.type)
        : (0, node_js_1.rootUnion)(l, r, ctx.type);
};
const parseArrayTuple = (def, scope) => (0, node_js_1.toArrayNode)((0, definition_js_1.parseDefinition)(def[0], scope));
const writeMalformedFunctionalExpressionMessage = (operator, rightDef) => `Expression requires a function following '${operator}' (was ${typeof rightDef})`;
exports.writeMalformedFunctionalExpressionMessage = writeMalformedFunctionalExpressionMessage;
const isIndexOneExpression = (def) => indexOneParsers[def[1]] !== undefined;
const indexOneParsers = {
    "|": parseBranchTuple,
    "&": parseBranchTuple,
    "[]": parseArrayTuple,
    "=>": narrow_js_1.parseNarrowTuple,
    "|>": morph_js_1.parseMorphTuple,
    ":": config_js_1.parseConfigTuple
};
const prefixParsers = {
    keyof: keyof_js_1.parseKeyOfTuple,
    instanceof: (def) => {
        if (typeof def[1] !== "function") {
            return (0, errors_js_1.throwParseError)(`Expected a constructor following 'instanceof' operator (was ${typeof def[1]}).`);
        }
        return { object: { class: def[1] } };
    },
    "===": (def) => ({ [(0, domains_js_1.domainOf)(def[1])]: { value: def[1] } }),
    node: (def) => def[1]
};
const isIndexZeroExpression = (def) => prefixParsers[def[0]] !== undefined;
