"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.writeBadDefinitionTypeMessage = exports.as = exports.parseDefinition = void 0;
const type_js_1 = require("../scopes/type.js");
const domains_js_1 = require("../utils/domains.js");
const errors_js_1 = require("../utils/errors.js");
const objectKinds_js_1 = require("../utils/objectKinds.js");
const serialize_js_1 = require("../utils/serialize.js");
const tuple_js_1 = require("./ast/tuple.js");
const record_js_1 = require("./record.js");
const string_js_1 = require("./string/string.js");
const parseDefinition = (def, ctx) => {
    const domain = (0, domains_js_1.domainOf)(def);
    if (domain === "string") {
        return (0, string_js_1.parseString)(def, ctx);
    }
    if (domain !== "object") {
        return (0, errors_js_1.throwParseError)((0, exports.writeBadDefinitionTypeMessage)(domain));
    }
    const objectKind = (0, objectKinds_js_1.objectKindOf)(def);
    switch (objectKind) {
        case "Object":
            return (0, record_js_1.parseRecord)(def, ctx);
        case "Array":
            return (0, tuple_js_1.parseTuple)(def, ctx);
        case "RegExp":
            return { string: { regex: def.source } };
        case "Function":
            if ((0, type_js_1.isType)(def)) {
                return ctx.type.scope.addAnonymousTypeReference(def, ctx);
            }
            if (isThunk(def)) {
                const returned = def();
                if ((0, type_js_1.isType)(returned)) {
                    return ctx.type.scope.addAnonymousTypeReference(returned, ctx);
                }
            }
            return (0, errors_js_1.throwParseError)((0, exports.writeBadDefinitionTypeMessage)("Function"));
        default:
            return (0, errors_js_1.throwParseError)((0, exports.writeBadDefinitionTypeMessage)(objectKind ?? (0, serialize_js_1.stringify)(def)));
    }
};
exports.parseDefinition = parseDefinition;
exports.as = Symbol("as");
const isThunk = (def) => typeof def === "function" && def.length === 0;
const writeBadDefinitionTypeMessage = (actual) => `Type definitions must be strings or objects (was ${actual})`;
exports.writeBadDefinitionTypeMessage = writeBadDefinitionTypeMessage;
