"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.writeUnterminatedEnclosedMessage = exports.enclosingChar = exports.parseEnclosed = void 0;
const regex_js_1 = require("../../../../nodes/rules/regex.js");
const parseEnclosed = (s, enclosing) => {
    const token = s.scanner.shiftUntil(untilLookaheadIsClosing[enclosing]);
    if (s.scanner.lookahead === "") {
        return s.error((0, exports.writeUnterminatedEnclosedMessage)(token, enclosing));
    }
    // Shift the scanner one additional time for the second enclosing token
    if (s.scanner.shift() === "/") {
        // Cache the regex instance to throw right way if its invalid
        (0, regex_js_1.getRegex)(token);
        s.setRoot({ string: { regex: token } });
    }
    else {
        s.setRoot({ string: { value: token } });
    }
};
exports.parseEnclosed = parseEnclosed;
exports.enclosingChar = {
    "'": 1,
    '"': 1,
    "/": 1
};
const untilLookaheadIsClosing = {
    "'": (scanner) => scanner.lookahead === `'`,
    '"': (scanner) => scanner.lookahead === `"`,
    "/": (scanner) => scanner.lookahead === `/`
};
const enclosingCharDescriptions = {
    '"': "double-quote",
    "'": "single-quote",
    "/": "forward slash"
};
const writeUnterminatedEnclosedMessage = (fragment, enclosing) => `${enclosing}${fragment} requires a closing ${enclosingCharDescriptions[enclosing]}`;
exports.writeUnterminatedEnclosedMessage = writeUnterminatedEnclosedMessage;
