"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.writeExpressionExpectedMessage = exports.writeMissingRightOperandMessage = exports.writeMissingOperandMessage = exports.writeUnresolvableMessage = exports.parseUnenclosed = void 0;
const numericLiterals_js_1 = require("../../../../utils/numericLiterals.js");
const parseUnenclosed = (s) => {
    const token = s.scanner.shiftUntilNextTerminator();
    s.setRoot(unenclosedToNode(s, token));
};
exports.parseUnenclosed = parseUnenclosed;
const unenclosedToNode = (s, token) => {
    if (s.ctx.type.scope.addParsedReferenceIfResolvable(token, s.ctx)) {
        return token;
    }
    return (maybeParseUnenclosedLiteral(token) ??
        s.error(token === ""
            ? (0, exports.writeMissingOperandMessage)(s)
            : (0, exports.writeUnresolvableMessage)(token)));
};
const maybeParseUnenclosedLiteral = (token) => {
    const maybeNumber = (0, numericLiterals_js_1.tryParseWellFormedNumber)(token);
    if (maybeNumber !== undefined) {
        return { number: { value: maybeNumber } };
    }
    const maybeBigint = (0, numericLiterals_js_1.tryParseWellFormedBigint)(token);
    if (maybeBigint !== undefined) {
        return {
            bigint: {
                value: maybeBigint
            }
        };
    }
};
const writeUnresolvableMessage = (token) => `'${token}' is unresolvable`;
exports.writeUnresolvableMessage = writeUnresolvableMessage;
const writeMissingOperandMessage = (s) => {
    const operator = s.previousOperator();
    return operator
        ? (0, exports.writeMissingRightOperandMessage)(operator, s.scanner.unscanned)
        : (0, exports.writeExpressionExpectedMessage)(s.scanner.unscanned);
};
exports.writeMissingOperandMessage = writeMissingOperandMessage;
const writeMissingRightOperandMessage = (token, unscanned) => `Token '${token}' requires a right operand${unscanned ? ` before '${unscanned}'` : ""}`;
exports.writeMissingRightOperandMessage = writeMissingRightOperandMessage;
const writeExpressionExpectedMessage = (unscanned) => `Expected an expression${unscanned ? ` before '${unscanned}'` : ""}`;
exports.writeExpressionExpectedMessage = writeExpressionExpectedMessage;
