"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Scanner = void 0;
class Scanner {
    chars;
    i;
    finalized = false;
    constructor(def) {
        this.chars = [...def];
        this.i = 0;
    }
    /** Get lookahead and advance scanner by one */
    shift() {
        return (this.chars[this.i++] ?? "");
    }
    get lookahead() {
        return (this.chars[this.i] ?? "");
    }
    shiftUntil(condition) {
        let shifted = "";
        while (this.lookahead) {
            if (condition(this, shifted)) {
                if (shifted[shifted.length - 1] === Scanner.escapeToken) {
                    shifted = shifted.slice(0, -1);
                }
                else {
                    break;
                }
            }
            shifted += this.shift();
        }
        return shifted;
    }
    shiftUntilNextTerminator() {
        this.shiftUntil(Scanner.lookaheadIsNotWhitespace);
        return this.shiftUntil(Scanner.lookaheadIsTerminator);
    }
    get unscanned() {
        return this.chars.slice(this.i, this.chars.length).join("");
    }
    lookaheadIs(char) {
        return this.lookahead === char;
    }
    lookaheadIsIn(tokens) {
        return this.lookahead in tokens;
    }
}
exports.Scanner = Scanner;
(function (Scanner) {
    Scanner.lookaheadIsTerminator = (scanner) => scanner.lookahead in Scanner.terminatingChars;
    Scanner.lookaheadIsNotWhitespace = (scanner) => scanner.lookahead !== Scanner.whiteSpaceToken;
    Scanner.comparatorStartChars = {
        "<": true,
        ">": true,
        "=": true
    };
    Scanner.terminatingChars = {
        ...Scanner.comparatorStartChars,
        "|": true,
        "&": true,
        ")": true,
        "[": true,
        "%": true,
        " ": true
    };
    Scanner.comparators = {
        "<": true,
        ">": true,
        "<=": true,
        ">=": true,
        "==": true
    };
    Scanner.oneCharComparators = {
        "<": true,
        ">": true
    };
    Scanner.comparatorDescriptions = {
        "<": "less than",
        ">": "more than",
        "<=": "at most",
        ">=": "at least",
        "==": "exactly"
    };
    Scanner.invertedComparators = {
        "<": ">",
        ">": "<",
        "<=": ">=",
        ">=": "<=",
        "==": "=="
    };
    Scanner.branchTokens = {
        "|": true,
        "&": true
    };
    Scanner.escapeToken = "\\";
    Scanner.whiteSpaceToken = " ";
})(Scanner = exports.Scanner || (exports.Scanner = {}));
