"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.isAnonymousName = exports.isType = exports.initializeType = void 0;
const traverse_js_1 = require("../traverse/traverse.js");
const chainableNoOpProxy_js_1 = require("../utils/chainableNoOpProxy.js");
const initializeType = (name, definition, config, scope) => {
    const root = {
        // temporarily initialize node/flat to aliases that will be included in
        // the final type in case of cyclic resolutions
        node: name,
        flat: [["alias", name]],
        allows: (data) => !namedTraverse(data).problems,
        assert: (data) => {
            const result = namedTraverse(data);
            return result.problems ? result.problems.throw() : result.data;
        },
        infer: chainableNoOpProxy_js_1.chainableNoOpProxy,
        inferIn: chainableNoOpProxy_js_1.chainableNoOpProxy,
        qualifiedName: (0, exports.isAnonymousName)(name)
            ? scope.getAnonymousQualifiedName(name)
            : `${scope.name}.${name}`,
        definition,
        scope,
        includesMorph: false,
        config
        // the "as" symbol from inferred is not used at runtime, so we check
        // that the rest of the type is correct then cast it
    };
    // define within a key to dynamically assign a name to the function
    const namedTraverse = {
        [name]: (data) => (0, traverse_js_1.traverseRoot)(namedTraverse, data)
    }[name];
    const t = Object.assign(namedTraverse, root);
    return t;
};
exports.initializeType = initializeType;
const isType = (value) => value?.infer === chainableNoOpProxy_js_1.chainableNoOpProxy;
exports.isType = isType;
const isAnonymousName = (name) => name[0] === "λ";
exports.isAnonymousName = isAnonymousName;
