"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getExactConstructorObjectKind = exports.objectKindDescriptions = exports.isArray = exports.hasObjectKind = exports.objectKindOf = exports.defaultObjectKinds = void 0;
const domains_js_1 = require("./domains.js");
const generics_js_1 = require("./generics.js");
// Built-in object constructors based on a subset of:
// https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects
exports.defaultObjectKinds = {
    Array,
    Date,
    Error,
    Function,
    Map,
    RegExp,
    Set,
    Object,
    String,
    Number,
    Boolean,
    WeakMap,
    WeakSet,
    Promise
};
const objectKindOf = (data, kinds) => {
    if ((0, domains_js_1.domainOf)(data) !== "object") {
        return undefined;
    }
    const kindSet = kinds ?? exports.defaultObjectKinds;
    let prototype = Object.getPrototypeOf(data);
    while (prototype?.constructor &&
        (!kindSet[prototype.constructor.name] ||
            !(data instanceof kindSet[prototype.constructor.name]))) {
        prototype = Object.getPrototypeOf(prototype);
    }
    return prototype?.constructor?.name;
};
exports.objectKindOf = objectKindOf;
const hasObjectKind = (data, kind, kinds) => (0, exports.objectKindOf)(data, kinds) === kind;
exports.hasObjectKind = hasObjectKind;
const isArray = (data) => Array.isArray(data);
exports.isArray = isArray;
/** Each defaultObjectKind's completion for the phrase "Must be _____" */
exports.objectKindDescriptions = {
    Object: "an object",
    Array: "an array",
    Function: "a function",
    Date: "a Date",
    RegExp: "a RegExp",
    Error: "an Error",
    Map: "a Map",
    Set: "a Set",
    String: "a String object",
    Number: "a Number object",
    Boolean: "a Boolean object",
    Promise: "a Promise",
    WeakMap: "a WeakMap",
    WeakSet: "a WeakSet"
};
// this will only return an object kind if it's the root constructor
// example TypeError would return undefined not 'Error'
const getExactConstructorObjectKind = (constructor) => {
    const constructorName = Object(constructor).name;
    return constructorName &&
        (0, generics_js_1.isKeyOf)(constructorName, exports.defaultObjectKinds) &&
        exports.defaultObjectKinds[constructorName] === constructor
        ? constructorName
        : undefined;
};
exports.getExactConstructorObjectKind = getExactConstructorObjectKind;
