"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.deserializePrimitive = exports.serializePrimitive = exports.stringify = exports.snapshot = void 0;
const domains_js_1 = require("./domains.js");
const generics_js_1 = require("./generics.js");
const numericLiterals_js_1 = require("./numericLiterals.js");
const snapshot = (data, opts = {}) => serializeRecurse(data, opts, []);
exports.snapshot = snapshot;
const stringify = (data, indent) => {
    switch ((0, domains_js_1.domainOf)(data)) {
        case "object":
            return JSON.stringify(serializeRecurse(data, stringifyOpts, []), null, indent);
        case "symbol":
            return stringifyOpts.onSymbol(data);
        default:
            return (0, exports.serializePrimitive)(data);
    }
};
exports.stringify = stringify;
const stringifyOpts = {
    onCycle: () => "(cycle)",
    onSymbol: (v) => `(symbol${v.description && ` ${v.description}`})`,
    onFunction: (v) => `(function${v.name && ` ${v.name}`})`
};
const serializeRecurse = (data, context, seen) => {
    switch ((0, domains_js_1.domainOf)(data)) {
        case "object":
            if (typeof data === "function") {
                return stringifyOpts.onFunction(data);
            }
            if (seen.includes(data)) {
                return "(cycle)";
            }
            const nextSeen = [...seen, data];
            if (Array.isArray(data)) {
                return data.map((item) => serializeRecurse(item, context, nextSeen));
            }
            const result = {};
            for (const k in data) {
                result[k] = serializeRecurse(data[k], context, nextSeen);
            }
            return result;
        case "symbol":
            return stringifyOpts.onSymbol(data);
        case "bigint":
            return `${data}n`;
        case "undefined":
            return "undefined";
        default:
            return data;
    }
};
const serializePrimitive = (value) => (typeof value === "string"
    ? `'${value}'`
    : typeof value === "bigint"
        ? `${value}n`
        : `${value}`);
exports.serializePrimitive = serializePrimitive;
const deserializePrimitive = (serialized) => ((0, generics_js_1.isKeyOf)(serialized, serializedKeywords)
    ? serializedKeywords[serialized]
    : serialized[0] === "'"
        ? serialized.slice(1, -1)
        : (0, numericLiterals_js_1.tryParseWellFormedBigint)(serialized) ??
            (0, numericLiterals_js_1.tryParseWellFormedNumber)(serialized, true));
exports.deserializePrimitive = deserializePrimitive;
const serializedKeywords = {
    true: true,
    false: false,
    undefined,
    null: null
};
